/*
 *  Project: NICE/2
 *
 *  By Sharon Dagan, idagan@il.ibm.com
 *  (C) copyright 1998, all rights reserved to IBM Israel
 *
 *  $Id$
 *  $Date$
 *
 **/

#define INCL_DOS
#define INCL_DOSERRORS
#define INCL_WIN
#include <os2.h>
#include <stdio.h>
#include <string.h>
#include "..\engine\globals.h"

#include <stdlib.h>

#define MEMPOOL 262144

extern BOOL EXPENTRY InputHook( HAB hab, PQMSG pqmsg, ULONG fs );  // from NICEDHK.DLL

VOID   message( APIRET rc );
APIRET install_tlw( VOID );
APIRET remove_tlw( VOID );
APIRET manage_hook( BOOL what );
VOID   cleanup( VOID );


int main( int argc, char ** argv )
{
	APIRET 		rc;
	
	if( argc == 2 )
		if( !stricmp( argv[ 1 ], "-start" ) )
			rc = install_tlw();
		else if( !stricmp( argv[ 1 ], "-stop" ) )
			rc = remove_tlw();
		else
			rc = ERR_USAGE;
	else
		rc = ERR_USAGE;
		
	message( rc );
	return rc;
};


VOID message( APIRET rc )
{
	char		msg[ 256 ];
	
	switch( rc )
	{
		case OK:
			return;
			
		case ERR_USAGE:
			printf( "NICE/2 by Sharon Dagan <idagan@il.ibm.com>\n" \
			        "usage: nicetlw.exe [ -start | -stop ]\n" );
			return;
			
		case ERR_TLW_STARTED:
			strcpy( msg, "nice tlw engine already started\n" );
			break;

		case ERR_TLW_STOPPED:
			strcpy( msg, "nice tlw engine not started\n" );
			break;
			
		case ERR_WINSETHOOK:
			strcpy( msg, "WinSetHook() faild\n" );
			break;	
			
		case ERR_WINSUBCLASSWINDOW:
			strcpy( msg, "WinSubclassWindow() faild\n" );
			break;			
			
		case ERR_TLW_HWND_NOT_FOUND:
			strcpy( msg, "TLW HWND not found\n" );
			break;
		
		default:
			strcpy( msg, "returned from an OS/2 API call\n" );
	};
	
	printf( "rc = %d %s", rc, msg );
};


APIRET install_tlw( VOID )
{
	return manage_hook( TRUE );
};	


APIRET remove_tlw( VOID )
{
	return manage_hook( FALSE );
};


/*
 *  install the system wide hook so we can get into PM context. After hook is in place
 *  we signal it what to do.
 *
 **/

APIRET manage_hook( BOOL what )
{
	APIRET 		rc;
	ULONG		ulRC;
	
	PTIB		ptib;
	PPIB		ppib;
	
	HAB			hab;
	HMQ			hmq;
	
	HMODULE		hm;
	CHAR		pszLoadError[ 256 ];
	
	HQUEUE      hqInstHookDone; // for RC queue
	REQUESTDATA	rd;
	PVOID		pBuf;
	ULONG		cbData;
	BYTE		bPriority;
	HEV			hevInstHookDone;

    ULONG       cbItems    = 0,
                ulBufSize  = 0;
    PSWBLOCK    pswblk     = NULL;
    PVOID       pvSwList   = NULL;
    HWND		hwndTlw;
    int			i;


	/*
	 *  mark this process as PM and initialize
	 *
	 **/

	rc = DosGetInfoBlocks( &ptib, &ppib );
	ASSERT( !rc );
	ppib->pib_ultype = SSF_TYPE_PM;
	
	hab = WinInitialize( 0 );
	ASSERT( hab );
	hmq = WinCreateMsgQueue( hab, 0 );
	ASSERT( hmq );
	
	/*
	 *  queue to get the return code from the hook
	 *
	 **/

	rc = DosCreateQueue( &hqInstHookDone,
						 QUE_FIFO,
						 INST_HOOK_QUEUE );
	ASSERT( !rc );					   		
	
	rc = DosCreateEventSem( NULL,
							&hevInstHookDone,
							DC_SEM_SHARED,
							FALSE );
	ASSERT( !rc );	
	
	/*
	 *  register exit list in case of troubles with the hook
	 *
	 **/
	
	rc = DosExitList( EXLST_ADD, ( PFNEXITLIST ) cleanup );
	ASSERT( !rc );
	if( rc )
		return rc;

	/*
	 *  get the HWND of the TLW - it's the entry with PID = 0
	 *
	 **/

    rc = DosAllocMem( &pvSwList, MEMPOOL, PAG_READ | PAG_WRITE );
    rc = DosSubSetMem( pvSwList, DOSSUB_INIT | DOSSUB_SPARSE_OBJ, MEMPOOL );

    cbItems = WinQuerySwitchList( 0, NULL, 0 );
    ulBufSize = ( cbItems * sizeof( SWENTRY ) ) + sizeof( HSWITCH );

    rc = DosSubAllocMem( pvSwList, ( PVOID ) &pswblk, ulBufSize );
    cbItems = WinQuerySwitchList( 0, pswblk, ulBufSize );

	hwndTlw = 0;
	for( i = 0; i < cbItems; i++ )
		if( pswblk->aswentry[ i ].swctl.idProcess == 0 )
		{
			hwndTlw = pswblk->aswentry[ i ].swctl.hwnd;
			break;
		};
		
	DosFreeMem( pvSwList );
    ASSERT( hwndTlw );
    if( !hwndTlw )
    	return ERR_TLW_HWND_NOT_FOUND;

	/*
	 *  load the hook dll
	 *
	 **/
	
	rc = DosLoadModule( pszLoadError,
	      		        sizeof( pszLoadError ),
   				        NICE_HOOK,
   				        &hm );
	if( rc )
		return rc;
	
	rc = WinSetHook( hab,
                 	 NULLHANDLE,
                     HK_INPUT,
                     ( PFN ) InputHook,
                     hm );
	if( !rc )
		return ERR_WINSETHOOK;

	/*
	 *  TLW
	 *
	 **/

    WinPostMsg( hwndTlw,
                WM_NULL,
                ( MPARAM ) ( what ? NICE_TLW_START : NICE_TLW_STOP ),
                ( MPARAM ) 0 );

	rc = DosReadQueue( hqInstHookDone,
				   	   &rd,
				       &cbData,
				       ( PPVOID ) &pBuf,
				       0,
				       DCWW_NOWAIT,
				       &bPriority,
				       hevInstHookDone );	
				
	if( rc == 0 )
		ulRC = rd.ulData;				
	else
	{			
		rc = DosWaitEventSem( hevInstHookDone, INST_HOOK_TIMEOUT );
		if( rc == 0 )
		{
			rc = DosReadQueue( hqInstHookDone,
				   	  		   &rd,
				               &cbData,
				               ( PPVOID ) &pBuf,
				               0,
				               DCWW_NOWAIT,
				               &bPriority,
				               hevInstHookDone );	

			ulRC = rd.ulData;
		}
		else
			ulRC = rc;		
	};

	rc = DosCloseEventSem( hevInstHookDone );	
	rc = DosCloseQueue( hqInstHookDone );					
	
	/*
	 *  relase the hook
	 *
	 **/
	
    WinReleaseHook( hab,
                    NULLHANDLE,
                    HK_INPUT,
                    ( PFN ) InputHook,
                    hm );

    WinBroadcastMsg( HWND_DESKTOP,
                     WM_NULL,
                     0,
                     0,
                     BMSG_FRAMEONLY | BMSG_POST );

    rc = DosFreeModule( hm );

	rc = DosExitList( EXLST_REMOVE, ( PFNEXITLIST ) cleanup );
	
	return ulRC;
};


/*
 *  Make sure we have a proper clean up so we wont get stuck with the hook
 *  DLL loaded
 *
 **/

VOID cleanup( VOID )
{
	HAB			hab;
	HMODULE		hm;
	
	hab = WinInitialize( 0 ); 	// second time will only get the HAB
	DosQueryModuleHandle( NICE_HOOK, &hm );
	
    WinReleaseHook( hab,
                    NULLHANDLE,
                    HK_INPUT  ,
                    ( PFN ) InputHook,
                    hm );

    WinBroadcastMsg( HWND_DESKTOP,
                     WM_NULL,
                     0,
                     0,
                     BMSG_FRAMEONLY | BMSG_POST );

	DosFreeModule( hm );
	WinTerminate( hab );
	
	DosExitList( EXLST_EXIT, ( PFNEXITLIST ) NULL );
};

